import os
import platform
import psutil
import re
import sys
import time

__all__ = [
           'SHORT_SIMPLE_TEST',
           'SST_DICT',
           'EPS',
           'UTF8',
           'PIRATES',
           'numeric_finder',
           'file_name',
           'get_passes',
           'n_extract',
           'c_prep',
           'MyNumber',
           'ExplodingNumber',
           'NotImplementedNumber',
           'set_high_priority',
           'detect_version',
           'trace',
           'success',
           'open_devnull',
           'close_devnull',
           'OutputUTF8',
          ]

### Tested entities

## Constants

# Restriction for simple_tests_signum.py; set it MANUALLY
SHORT_SIMPLE_TEST = 3 # 0: pass only tests common with v1.0.2 to compare timing
                      # 1: pass only tests common with v1.1.0 (this includes v1.0.2)
                      # 2: pass only tests common with v1.2.2
                      # 3: pass all tests (current v1.2.3)
# Outputs of detect_version() vs restrictions
SST_DICT          = {'1.0.2': 0, '1.1.0+': 1, '1.2.0': 2, '1.2.1': 2, '1.2.2': 2, '1.2.3': 3, 'default': 2,}

EPS = 1e-9

UTF8 = 'utf-8'

MAX_PASSES = {
    'simple_test_signum': 10000,
    '41_tests_signum':   100000,
    '57_tests_signum':   100000,
    'fastsign_41_tests': 100000,
    'fastsign_57_tests': 100000,
    'leak_test':        1000000,
    'default':             1000,
}

PIRATES = "☠️15 men on the dead man's chest☠️"

numeric_finder = re.compile(r"[-+]?(?:\d+\.\d*|\.\d+|\d+)(?:[eE][-+]?\d+)?")

## Functions

# Get file name from path
def file_name(file_path):
    return os.path.splitext(os.path.basename(file_path))[0]

# Select MAX_PASSES for a test by file name
def get_passes(file_path):
    return MAX_PASSES.get(file_name(file_path), MAX_PASSES['default'])

## Functions used to preprocess

# Number extraction from str for preprocess
def n_extract(s):
    if isinstance(s, str):
        match = numeric_finder.search(s)
        return (float(match.group()),) if match else None
    return None

# sign(complex) with recursion for preprocess
def c_prep(z):
    if z == 0 or not isinstance(z, complex): return None
    # complex z != 0
    return (0, z/abs(z))

## Custom comparable classes

class MyNumber: # __float__ is absent
    def __init__(self, value):
        self.value = value
    def __gt__(self, other):
        return self.value > other
    def __lt__(self, other):
        return self.value < other
    def __eq__(self, other):
        return self.value == other
    def __repr__(self):
        return f'MyNumber({self.value})'

class ExplodingNumber: # Instead of reach comprisons, raises exceptions; __float__ explodes also
    def __init__(self, value):
        self.value = value
    def __gt__(self, other):
        raise RuntimeError("Boom!")
    def __lt__(self, other):
        raise RuntimeError("Boom!")
    def __eq__(self, other):
        raise RuntimeError("Boom!")
    def __float__(self):
        raise RuntimeError("Boom!")
    def __repr__(self):
        return f'ExplodingNumber({self.value})'

class NotImplementedNumber: # Stubs for future implemetation; __float__ is implemented
    def __init__(self, value):
        self.value = value
    def __gt__(self, other):
        return NotImplemented
    def __lt__(self, other):
        return NotImplemented
    def __eq__(self, other):
        return NotImplemented
    def __float__(self):
        return float(self.value)
    def __repr__(self):
        return f'NotImplementedNumber({self.value})'

### Service entities

# High process priority for best benchmarking
def set_high_priority():
    try:
        p = psutil.Process(pid := os.getpid())
        if (syst := platform.system()) == "Windows":
            p.nice(psutil.HIGH_PRIORITY_CLASS)
        else: # Linux/macOS
            p.nice(-10) # Priority -10 needs rights!
        return f'{syst}: high priority for process {pid} was successfully set'
    except Exception as e: # Not enough rights, or something else went wrong: ignore
        return f'{syst}: cannot set high priority for  process {pid}, got `{str(e)}`'

# Detecting version
def detect_version():
    if not (signum_mod := sys.modules.get('signum')): # 'signum' not imported
        raise ImportError(
            "Module 'signum' not found in sys.modules\n"
            "Make sure you have 'from signum import sign' in your test script "
            "before calling 'success()' or 'detect_version()'"
        )
    if hasattr(signum_mod, '__version__'): # v1.2.0+
        return signum_mod.__version__
    try:
        signum_mod.sign(0, preprocess=None)
        return("1.1.0+")
    except TypeError:
        return("1.0.2")

## Tracing

def trace(pcnt, cnt, scnt, what=None):
    ending = 's'
    delta = cnt - pcnt
    if delta == 1: ending = ''
    beginning, descr = '   --- ', '' # trace without description is for simple_test_signum.py
    if what: beginning, descr = '', f': `{what}`'
    return f"{beginning}{delta:2} test{ending:1} for Sec. {scnt:2}{descr} passed, total {cnt:3} tests passed"

def success(counter, s_cnt=None, start_time=None, passes=None):
    version = detect_version()
    sections = ''
    if s_cnt: sections = f' from {s_cnt} sections'
    seconds = ''
    if start_time: seconds = f' in {time.perf_counter() - start_time:>9.4f}s'
    reps = ''
    if passes: reps = f' (passes: {passes}; sign calls: {passes*counter})'
    return f'Success of v{version:<6}: {counter} tests{sections} passed{seconds}{reps}.'

## os.devnull

def open_devnull():
    return open(os.devnull, 'w', encoding=UTF8)

def close_devnull(f):
    if f:
        try:
            f.flush()
        finally:
            f.close()

# Class to switch sys.stdout and sys.stderr
class OutputUTF8:
    def __init__(self):
        self.original_stdout_params = {'encoding': sys.stdout.encoding, 'errors': sys.stdout.errors}
        self.original_stderr_params = {'encoding': sys.stderr.encoding, 'errors': sys.stderr.errors}

    def set_utf8(self):
        # Switch sys.stdout and sys.stderr to 'utf-8' encoding
        sys.stdout.flush(); sys.stdout.reconfigure(encoding=UTF8)
        sys.stderr.flush(); sys.stderr.reconfigure(encoding=UTF8)

    def reset_from_utf8(self):
        # Restore stdout and stderr
        sys.stdout.flush(); sys.stdout.reconfigure(**self.original_stdout_params)
        sys.stderr.flush(); sys.stderr.reconfigure(**self.original_stderr_params)



from math import nan as nan, isnan as isnan

def fastsign(x):
    err = None;
    try:
        if x >  0:   return  1
        if x <  0:   return -1
        if x == 0:   return  0
        if isnan(x): return nan
        raise TypeError(f"Unusual results with type {type(x)}")
    except (TypeError, ValueError, NotImplementedError) as e:
        err = e
        if not isinstance(x, str):
            try:
                d = float(x)
                if isnan(d): return nan
                if d == 0.0: return  0
                if d >  0.0: return  1
                if d <  0.0: return -1
            except (TypeError, ValueError) as e:
               err = e

    raise TypeError(
              f"Python fastsign({repr(x)}) for type {type(x)}: cannot compare or check for NaN"
          ) from err

__all__ = [fastsign,]



from signum import sign
from testing import get_passes, MyNumber as _MyNumber, set_high_priority, success, OutputUTF8

from decimal import Decimal
from fractions import Fraction
from math import nan, inf
import sympy
import time

MAX_PASSES = get_passes(__file__)

# Switch sys.stdout and sys.stderr to 'utf-8' encoding
outflows = OutputUTF8()
outflows.set_utf8()

start_time = None

print(f'***** Test: {__file__}')
print(f'MAX_PASSES: {MAX_PASSES}')
print(f'*** {set_high_priority()} ***\n')
for _ in range(MAX_PASSES + 1):
    (sign(-5))
    (sign(-1))
    (sign(0))
    (sign(1))
    (sign(5))
    (sign(True))
    (sign(False))
    (sign(10**1000))
    (sign(-10**1000))
    (sign(10**1000-10**1000))
    (sign(-5.0))
    (sign(-1.0))
    (sign(0.0))
    (sign(1.0))
    (sign(5.0))
    (sign(float('-0.0')))
    (sign(float('+0.0')))
    (sign(-inf))
    (sign(inf))
    (sign(float('-nan')))
    (sign(nan))
    (sign(0.0*nan))
    (sign(Fraction(-5, 2)))
    (sign(Fraction(-1, 2)))
    (sign(Fraction(0, 2)))
    (sign(Fraction(1, 2)))
    (sign(Fraction(5, 2)))
    (sign(Decimal(-5.5)))
    (sign(Decimal(-1.5)))
    (sign(Decimal(0.0)))
    (sign(Decimal(1.5)))
    (sign(Decimal(5.5)))
    (sign(Decimal('NaN')))
    x_sym = sympy.Symbol('x')
    expr = x_sym
    val = expr.subs(x_sym, -3.14)
    (sign(val))
    (sign(sympy.Rational(3, 4)))
    (sign(sympy.nan))
    (sign(_MyNumber(-5)))
    (sign(_MyNumber(-1)))
    (sign(_MyNumber(0)))
    (sign(_MyNumber(1)))
    (sign(_MyNumber(5.1)))

    if start_time is None: # The very first pass to warm Python
        start_time = time.perf_counter()

print(f'{success(41, s_cnt=None, start_time=start_time, passes=MAX_PASSES)}\n')

# Restore stdout and stderr
outflows.reset_from_utf8()



from signum import sign, fastsign

from testing import EPS, PIRATES, n_extract, c_prep, \
                    MyNumber, ExplodingNumber, NotImplementedNumber, trace, success, OutputUTF8

from decimal import Decimal
from fractions import Fraction
from math import nan, isnan, inf
import sympy
import unittest

class TestSignum(unittest.TestCase):

    def test_sign(self):
        self.buffer = []
        s_cnt = 0
        counter = 0
        # --- int
        s_cnt += 1; prev_counter = counter
        self.assertEqual(sign(-5), -1); counter += 1
        self.assertEqual(sign(-1), -1); counter += 1
        self.assertEqual(sign(0), 0); counter += 1
        self.assertEqual(sign(1), 1); counter += 1
        self.assertEqual(sign(5), 1); counter += 1
        self.buffer.append(trace(prev_counter, counter, s_cnt, what="'int'"))

        # ------ bool
        s_cnt += 1; prev_counter = counter
        self.assertEqual(sign(True), 1); counter += 1
        self.assertEqual(sign(False), 0); counter += 1
        self.buffer.append(trace(prev_counter, counter, s_cnt, what="'bool'"))

        # ------ big numbers
        s_cnt += 1; prev_counter = counter
        self.assertEqual(sign(10**1000), 1); counter += 1
        self.assertEqual(sign(-10**1000), -1); counter += 1
        self.assertEqual(sign(10**1000-10**1000), 0); counter += 1
        self.buffer.append(trace(prev_counter, counter, s_cnt, what="big 'int'"))

        # --- float
        s_cnt += 1; prev_counter = counter
        self.assertEqual(sign(-5.0), -1); counter += 1
        self.assertEqual(sign(-1.0), -1); counter += 1
        self.assertEqual(sign(0.0), 0); counter += 1
        self.assertEqual(sign(1.0), 1); counter += 1
        self.assertEqual(sign(5.0), 1); counter += 1
        self.buffer.append(trace(prev_counter, counter, s_cnt, what="'float'"))

        # ------ -0.0 and +0.0
        s_cnt += 1; prev_counter = counter
        self.assertEqual(sign(float('-0.0')), 0); counter += 1
        self.assertEqual(sign(float('+0.0')), 0); counter += 1
        self.buffer.append(trace(prev_counter, counter, s_cnt, what="±0.0"))

        # ------ -inf and inf
        s_cnt += 1; prev_counter = counter
        self.assertEqual(sign(-inf), -1); counter += 1
        self.assertEqual(sign(inf), 1); counter += 1
        self.buffer.append(trace(prev_counter, counter, s_cnt, what="infinity"))

        # ------ -nan (the same as nan), nan
        s_cnt += 1; prev_counter = counter
        self.assertTrue(isnan(sign(float('-nan')))); counter += 1
        self.assertTrue(isnan(sign(nan))); counter += 1
        self.assertTrue(isnan(sign(0.0*nan))); counter += 1
        self.buffer.append(trace(prev_counter, counter, s_cnt, what="NaN"))

        # --- Fraction
        s_cnt += 1; prev_counter = counter
        self.assertEqual(sign(Fraction(-5, 2)), -1); counter += 1
        self.assertEqual(sign(Fraction(-1, 2)), -1); counter += 1
        self.assertEqual(sign(Fraction(0, 2)), 0); counter += 1
        self.assertEqual(sign(Fraction(1, 2)), 1); counter += 1
        self.assertEqual(sign(Fraction(5, 2)), 1); counter += 1
        self.buffer.append(trace(prev_counter, counter, s_cnt, what="'Fraction'"))

        # --- Tests with a very small argument kindly provided by Tim Peters
        s_cnt += 1; prev_counter = counter
        self.assertEqual(sign(Fraction(1, 1 << 2000)), 1); counter += 1
        self.assertEqual(sign(float(Fraction(1, 1 << 2000))), 0); counter += 1
        self.buffer.append(trace(prev_counter, counter, s_cnt, what="Very small argument (kindly provided by Tim Peters)"))

        # --- Decimal
        s_cnt += 1; prev_counter = counter
        self.assertEqual(sign(Decimal(-5.5)), -1); counter += 1
        self.assertEqual(sign(Decimal(-1.5)), -1); counter += 1
        self.assertEqual(sign(Decimal(0.0)), 0); counter += 1
        self.assertEqual(sign(Decimal(1.5)), 1); counter += 1
        self.assertEqual(sign(Decimal(5.5)), 1); counter += 1
        self.buffer.append(trace(prev_counter, counter, s_cnt, what="'Decimal'"))

        # ------ Decimal NaN
        s_cnt += 1; prev_counter = counter
        self.assertTrue(isnan(sign(Decimal('NaN')))); counter += 1
        self.buffer.append(trace(prev_counter, counter, s_cnt, what="Decimal NaN"))

        # --- sympy
        x_sym = sympy.Symbol('x')
        expr = x_sym
        val = expr.subs(x_sym, -3.14)
        s_cnt += 1; prev_counter = counter
        self.assertEqual(sign(val), -1); counter += 1
        self.assertEqual(sign(sympy.Rational(3, 4)), 1); counter += 1
        self.buffer.append(trace(prev_counter, counter, s_cnt, what="sympy"))

        # ------ sympy.nan
        s_cnt += 1; prev_counter = counter
        self.assertTrue(isnan(sign(sympy.nan))); counter += 1
        self.buffer.append(trace(prev_counter, counter, s_cnt, what="sympy.nan"))

        # --- New custom class (testing possible future extentions)
        #     This class has no __float__ that tests one subtle branch in the C++ code
        s_cnt += 1; prev_counter = counter
        self.assertEqual(sign(MyNumber(-5)), -1); counter += 1
        self.assertEqual(sign(MyNumber(-1)), -1); counter += 1
        self.assertEqual(sign(MyNumber(0)), 0); counter += 1
        self.assertEqual(sign(MyNumber(1)), 1); counter += 1
        self.assertEqual(sign(MyNumber(5)), 1); counter += 1
        with self.assertRaisesRegex(TypeError, r'signum\.sign: invalid argument `MyNumber\(nan\)`'):
            sign(MyNumber(nan))
        counter += 1
        self.buffer.append(trace(prev_counter, counter, s_cnt, what="new custom class"))

        # Testing inappropriate arguments and types (non-scalar, non-comparable, etc.)

        # --- Invalid number of positional arguments (0, 0 with keys, 2, 3, 4, 5); invalid keyword
        s_cnt += 1; prev_counter = counter
        with self.assertRaisesRegex(TypeError, r"signum\.sign\(\) takes 1 or 2 positional arguments, got 0"):
            sign()
        counter += 1
        with self.assertRaisesRegex(TypeError, r"signum\.sign\(\) takes 1 or 2 positional arguments, got 0"):
            sign(preprocess=lambda a: (float(a),), if_exc=None)
        counter += 1
        with self.assertRaisesRegex(TypeError, r'signum\.sign\(\) takes 1 or 2 positional arguments, got 3'):
            sign(-1, 0, 1)
        counter += 1
        with self.assertRaisesRegex(TypeError, r'signum\.sign\(\) takes 1 or 2 positional arguments, got 4'):
            sign(-1, 0, 1, 4)
        counter += 1
        with self.assertRaisesRegex(TypeError, r'signum\.sign\(\) takes 1 or 2 positional arguments, got 5'):
            sign(-1, 0, 1, 4, 5)
        counter += 1
        with self.assertRaisesRegex(TypeError, r"signum.sign\(\) got an unexpected keyword argument 'code_shift'"):
            sign(5.0, code_shift=2)
        counter += 1
        self.buffer.append(trace(prev_counter, counter, s_cnt, what="invalid number of arguments"))

        # --- ExplodingNumber, NotImplementedNumber
        s_cnt += 1; prev_counter = counter
        with self.assertRaisesRegex(TypeError, r'signum.sign: invalid argument `ExplodingNumber\(-3\.14\)`'):
            sign(ExplodingNumber(-3.14))
        counter += 1
        with self.assertRaisesRegex(TypeError, r'signum.sign: invalid argument `NotImplementedNumber\(-3\.14\)`'):
            sign(NotImplementedNumber(-3.14))
        counter += 1
        self.buffer.append(trace(prev_counter, counter, s_cnt, what="Exploding and Not Implemented numbers"))

        # --- None, str, complex, list, set
        tests = [(r"`None`", None), (r"`'5\.0'`", '5.0'), (r"`'nan'`", 'nan'),
                 (r"`'number 5'`", 'number 5'), (r"`\(-1\+1j\)`", -1+1j), (r"`\[-8\.75\]`", [-8.75]),
                 (r"`\{-3\.14\}`", {-3.14}),
                ]

        s_cnt += 1; prev_counter = counter
        for msg, obj in tests:
            with self.subTest(obj=obj):
                with self.assertRaisesRegex(TypeError,
                                            r'signum\.sign: invalid argument ' + msg):
                    sign(obj)
                counter += 1
        self.buffer.append(trace(prev_counter, counter, s_cnt, what="inappropriate types"))

        ## Testing additional key arguments (preprocess=, is_exc=, both, codeshift=, combinations)

        # --- preprocess key, simple argument replacement, string conversion
        s_cnt += 1; prev_counter = counter
        self.assertEqual(sign('5.0', preprocess=lambda a: (float(a),)), 1); counter += 1
        self.assertTrue(isnan(sign('nan', preprocess=lambda a: (float(a),)))); counter += 1
        self.assertEqual(sign(-18, preprocess=lambda a: (float(a),)), -1); counter += 1
        self.buffer.append(trace(prev_counter, counter, s_cnt, what="preprocess with simple str replacement"))

        # --- preprocess key, argument replacement, treat small number as zero
        s_cnt += 1; prev_counter = counter
        tests = [(-1, -1), (0, 0), (-.187e-17, 0), (5.0, 1)]
        for x, y in tests:
            self.assertEqual(sign(x, preprocess=lambda a: (0 if abs(a) < EPS else a,)), y);  counter += 1
        self.buffer.append(trace(prev_counter, counter, s_cnt, what="preprocess treating small number as zero"))

        # --- preprocess key, replace only string argument, extract number from string
        s_cnt += 1; prev_counter = counter
        self.assertEqual(sign(PIRATES, preprocess=n_extract), 1); counter += 1
        self.assertEqual(sign('Temperature is -.12e+02 °C', preprocess=n_extract), -1); counter += 1
        with self.assertRaisesRegex(TypeError, r"signum.sign: invalid argument `'error'` \(type 'str'\)"):
            sign('error', preprocess=n_extract)
        counter += 1
        self.assertEqual(sign(123, preprocess=n_extract), 1); counter += 1
        self.buffer.append(trace(prev_counter, counter, s_cnt, what="preprocess extracting numbers from str"))

        # --- preprocess key, replace result, permits complex arguments
        s_cnt += 1; prev_counter = counter
        tests = [(-1+1j, '(-0.7071067811865475+0.7071067811865475j)'), (-18.4, '-1')]
        for x, y in tests:
            self.assertEqual(str(sign(x, preprocess=c_prep)), y); counter += 1
        self.buffer.append(trace(prev_counter, counter, s_cnt, what="preprocess permitting complex arguments"))

        # --- preprocess key, replace result, float result for 'float' and 'Decimal', sign recursion
        s_cnt += 1; prev_counter = counter
        tests = [(-5, -1, int), (-5.0, -1.0, float), (Decimal(-5.5), -1.0, float)]
        for x, y, t in tests:
            self.assertEqual((s := sign(x, preprocess=lambda a: (a, float(sign(a))) if isinstance(a, (float, Decimal)) else None)), y)
            self.assertIsInstance(s, t); counter += 1
        self.buffer.append(trace(prev_counter, counter, s_cnt, what="preprocess with float result and sign recursion"))

        # --- preprocess key, replace result or argument, treat small number as zero differently
        s_cnt += 1; prev_counter = counter
        tests = [(-1, -1), (0, 0), (-.187e-17, 0), (5.0, 1)]
        ppl = lambda x: (x, 0) if abs(x) < EPS else (x,)
        for x, y in tests:
            self.assertEqual(sign(x, preprocess=ppl), y); counter += 1
        self.buffer.append(trace(prev_counter, counter, s_cnt, what="preprocess replacing result or argument"))

        # --- if_exc key (exception safety)
        s_cnt += 1; prev_counter = counter
        tests = [(None, 'None'), ('5.0', '-2'), ('nan', 'nan'), ('number 5', 'None'),
                 (-1+1j, 'None'), ([-8.75], '-2'), ({-3.14}, 'nan'), # blocking exceptions
                 (-1, '-1'), (31.4, '1'), (nan, 'nan'), (Fraction(-99, 19), '-1'), (Decimal('101.78'), '1'),]                                              # valid nuneric types
        flag = 0
        repl = [None, -2, nan, None,]
        nrepl = len(repl)
        for x, y in tests:
            self.assertEqual(repr(sign(x, if_exc=(repl[flag],))), y); counter += 1
            flag = (flag + 1) % nrepl
        self.buffer.append(trace(prev_counter, counter, s_cnt, what="if_exc key"))

        # --- both preprocess and if_exc key
        s_cnt += 1; prev_counter = counter
        tests = [(-5, '-1', int), (-5.0, '-1.0', float),
                 (Decimal(-5.5), '-1.0', float), ('error', 'None', type(None))]
        for x, y, t in tests:
            self.assertEqual(str(s := sign(x, preprocess=lambda a: (a, float(sign(a))) if isinstance(a, (float, Decimal)) else None, if_exc=(None,))), y)
            self.assertIsInstance(s, t); counter += 1
        self.buffer.append(trace(prev_counter, counter, s_cnt, what="both preprocess and if_exc"))

        # codeshift and combinations
        s_cnt += 1; prev_counter = counter
        self.assertEqual(sign('error', codeshift=2), 0); counter += 1
        self.assertEqual(sign(-5, codeshift=2), 1); counter += 1
        self.assertEqual(sign(0, codeshift=2), 2); counter += 1
        self.assertEqual(sign(5, codeshift=2), 3); counter += 1
        self.assertEqual(sign(nan, codeshift=2), 4); counter += 1
        self.assertEqual(sign('error', if_exc=(-7,), codeshift=2), -7); counter += 1
        self.assertEqual(sign(-5, if_exc=(-7,), codeshift=2), 1); counter += 1
        self.assertEqual(sign(0, if_exc=(-7,), codeshift=2), 2); counter += 1
        self.assertEqual(sign(5, if_exc=(-7,), codeshift=2), 3); counter += 1
        self.assertEqual(sign(nan, if_exc=(-7,), codeshift=2), 4); counter += 1
        self.assertEqual(sign('error', preprocess=lambda a: (0 if abs(a) < EPS else a,), codeshift=2), 0); counter += 1
        self.assertEqual(sign(-1, preprocess=lambda a: (0 if abs(a) < EPS else a,), codeshift=2), 1); counter += 1
        self.assertEqual(sign(0, preprocess=lambda a: (0 if abs(a) < EPS else a,), codeshift=2), 2); counter += 1
        self.assertEqual(sign(-1.87e-18, preprocess=lambda a: (0 if abs(a) < EPS else a,), codeshift=2), 2); counter += 1
        self.assertEqual(sign(5.0, preprocess=lambda a: (0 if abs(a) < EPS else a,), codeshift=2), 3); counter += 1
        self.assertEqual(sign('error', preprocess=lambda a: (a, float(sign(a))) if isinstance(a, float) else None, codeshift=1), -1); counter += 1
        self.assertEqual((s := sign(-5.0, preprocess=lambda a: (a, float(sign(a))) if isinstance(a, float) else None, codeshift=1)), -1.0)
        self.assertIsInstance(s, float); counter += 1
        self.assertEqual((s := sign(0.0, preprocess=lambda a: (a, float(sign(a))) if isinstance(a, float) else None, codeshift=1)), 0.0)
        self.assertIsInstance(s, float); counter += 1
        self.assertEqual(sign(0, preprocess=lambda a: (a, float(sign(a))) if isinstance(a, float) else None, codeshift=1), 1); counter += 1
        self.assertEqual(sign(5, preprocess=lambda a: (a, float(sign(a))) if isinstance(a, float) else None, codeshift=1), 2); counter += 1
        self.assertTrue(isnan(sign(nan, preprocess=lambda a: (a, float(sign(a))) if isinstance(a, float) else None, codeshift=1))); counter += 1
        self.assertEqual(sign('error', preprocess=lambda a: (a, float(sign(a))) if isinstance(a, float) else None, if_exc=(-7,), codeshift=1), -7); counter += 1
        self.assertEqual((s := sign(-5.0, preprocess=lambda a: (a, float(sign(a))) if isinstance(a, float) else None, if_exc=(-7,), codeshift=1)), -1.0)
        self.assertIsInstance(s, float); counter += 1
        self.assertEqual((s := sign(0.0, preprocess=lambda a: (a, float(sign(a))) if isinstance(a, float) else None, if_exc=(-7,), codeshift=1)), 0.0)
        self.assertIsInstance(s, float); counter += 1
        self.assertEqual(sign(0, preprocess=lambda a: (a, float(sign(a))) if isinstance(a, float) else None, if_exc=(-7,), codeshift=1), 1); counter += 1
        self.assertEqual(sign(5, preprocess=lambda a: (a, float(sign(a))) if isinstance(a, float) else None, if_exc=(-7,), codeshift=1), 2); counter += 1
        self.assertTrue(isnan(sign(nan, preprocess=lambda a: (a, float(sign(a))) if isinstance(a, float) else None, if_exc=(-7,), codeshift=1))); counter += 1
        self.buffer.append(trace(prev_counter, counter, s_cnt, what="codeshift and key combinations"))

        # v1.2.3: positional codeshift
        s_cnt += 1; prev_counter = counter
        with self.assertRaisesRegex(TypeError, r"signum\.sign\(\): the 2nd positional argument used; "
                                               r"'codeshift=' is not permitted"):
            sign(-77, None, codeshift=2)
        counter += 1
        self.assertEqual(sign('error', 2), 0); counter += 1
        self.assertEqual(sign(-5, 2), 1); counter += 1
        self.assertEqual(sign(0, 2), 2); counter += 1
        self.assertEqual(sign(5, 2), 3); counter += 1
        self.assertEqual(sign(nan, 2), 4); counter += 1
        self.assertEqual(sign('error', 2, if_exc=(-7,)), -7); counter += 1
        self.assertEqual(sign(-5, 2, if_exc=(-7,)), 1); counter += 1
        self.assertEqual(sign(0, 2, if_exc=(-7,)), 2); counter += 1
        self.assertEqual(sign(5, 2, if_exc=(-7,)), 3); counter += 1
        self.assertEqual(sign(nan, 2, if_exc=(-7,)), 4); counter += 1
        self.assertEqual(sign('error', 2, preprocess=lambda a: (0 if abs(a) < EPS else a,)), 0); counter += 1
        self.assertEqual(sign(-1, 2, preprocess=lambda a: (0 if abs(a) < EPS else a,)), 1); counter += 1
        self.assertEqual(sign(0, 2, preprocess=lambda a: (0 if abs(a) < EPS else a,)), 2); counter += 1
        self.assertEqual(sign(-1.87e-18, 2, preprocess=lambda a: (0 if abs(a) < EPS else a,)), 2); counter += 1
        self.assertEqual(sign(5.0, 2, preprocess=lambda a: (0 if abs(a) < EPS else a,)), 3); counter += 1
        self.assertEqual(sign('error', 1, preprocess=lambda a: (a, float(sign(a))) if isinstance(a, float) else None), -1); counter += 1
        self.assertEqual(s := sign(-5.0, 1, preprocess=lambda a: (a, float(sign(a))) if isinstance(a, float) else None), -1.0);
        self.assertIsInstance(s, float); counter += 1
        self.assertEqual(s := sign(0.0, 1, preprocess=lambda a: (a, float(sign(a))) if isinstance(a, float) else None), 0.0);
        self.assertIsInstance(s, float); counter += 1
        self.assertEqual(sign(0, 1, preprocess=lambda a: (a, float(sign(a))) if isinstance(a, float) else None), 1); counter += 1
        self.assertEqual(sign(5, 1, preprocess=lambda a: (a, float(sign(a))) if isinstance(a, float) else None), 2); counter += 1
        self.assertTrue(isnan(sign(nan, 1, preprocess=lambda a: (a, float(sign(a))) if isinstance(a, float) else None))); counter += 1
        self.assertEqual(sign(nan, 1, preprocess=lambda a: (a, float(sign(a))) if isinstance(a, float) and not isnan(a) else None), 3); counter += 1
        self.assertEqual(sign('error', 1, preprocess=lambda a: (a, float(sign(a))) if isinstance(a, float) else None, if_exc=(-7,)), -7); counter += 1
        self.assertEqual(s := sign(-5.0, 1, preprocess=lambda a: (a, float(sign(a))) if isinstance(a, float) else None, if_exc=(-7,)), -1.0);
        self.assertIsInstance(s, float); counter += 1
        self.assertEqual(s := sign(0.0, 1, preprocess=lambda a: (a, float(sign(a))) if isinstance(a, float) else None, if_exc=(-7,)), 0.0);
        self.assertIsInstance(s, float); counter += 1
        self.assertEqual(sign(0, 1, preprocess=lambda a: (a, float(sign(a))) if isinstance(a, float) else None, if_exc=(-7,)), 1); counter += 1
        self.assertEqual(sign(5, 1, preprocess=lambda a: (a, float(sign(a))) if isinstance(a, float) else None, if_exc=(-7,)), 2); counter += 1
        self.assertTrue(isnan(sign(nan, 1, preprocess=lambda a: (a, float(sign(a))) if isinstance(a, float) else None, if_exc=(-7,)))); counter += 1
        self.assertEqual(sign(nan, 1, preprocess=lambda a: (a, float(sign(a))) if isinstance(a, float) and not isnan(a) else None, if_exc=(-7,)), 3); counter += 1
        self.buffer.append(trace(prev_counter, counter, s_cnt, what="positional codeshift"))

        # v1.2.3: fastsign
        s_cnt += 1; prev_counter = counter
        self.assertEqual(fastsign(-5), -1); counter += 1
        self.assertEqual(fastsign(-1), -1); counter += 1
        self.assertEqual(fastsign(0), 0); counter += 1
        self.assertEqual(fastsign(1), 1); counter += 1
        self.assertEqual(fastsign(5), 1); counter += 1
        self.assertEqual(fastsign(True), 1); counter += 1
        self.assertEqual(fastsign(False), 0); counter += 1
        self.assertEqual(fastsign(10**1000), 1); counter += 1
        self.assertEqual(fastsign(-10**1000), -1); counter += 1
        self.assertEqual(fastsign(10**1000-10**1000), 0); counter += 1
        self.assertEqual(fastsign(-5.0), -1); counter += 1
        self.assertEqual(fastsign(-1.0), -1); counter += 1
        self.assertEqual(fastsign(0.0), 0); counter += 1
        self.assertEqual(fastsign(1.0), 1); counter += 1
        self.assertEqual(fastsign(5.0), 1); counter += 1
        self.assertEqual(fastsign(float('-0.0')), 0); counter += 1
        self.assertEqual(fastsign(float('+0.0')), 0); counter += 1
        self.assertEqual(fastsign(-inf), -1); counter += 1
        self.assertEqual(fastsign(inf), 1); counter += 1
        self.assertTrue(isnan(fastsign(float('-nan')))); counter += 1
        self.assertTrue(isnan(fastsign(nan))); counter += 1
        self.assertTrue(isnan(fastsign(0.0*nan))); counter += 1
        self.assertEqual(fastsign(Fraction(-5, 2)), -1); counter += 1
        self.assertEqual(fastsign(Fraction(-1, 2)), -1); counter += 1
        self.assertEqual(fastsign(Fraction(0, 2)), 0); counter += 1
        self.assertEqual(fastsign(Fraction(1, 2)), 1); counter += 1
        self.assertEqual(fastsign(Fraction(5, 2)), 1); counter += 1
        self.assertEqual(fastsign(Fraction(1, 1 << 2000)), 1); counter += 1
        self.assertEqual(fastsign(float(Fraction(1, 1 << 2000))), 0); counter += 1
        self.assertEqual(fastsign(Decimal(-5.5)), -1); counter += 1
        self.assertEqual(fastsign(Decimal(-1.5)), -1); counter += 1
        self.assertEqual(fastsign(Decimal(0.0)), 0); counter += 1
        self.assertEqual(fastsign(Decimal(1.5)), 1); counter += 1
        self.assertEqual(fastsign(Decimal(5.5)), 1); counter += 1
        self.assertTrue(isnan(fastsign(Decimal('NaN')))); counter += 1
        x_sym = sympy.Symbol('x')
        expr = x_sym
        val = expr.subs(x_sym, -3.14)
        self.assertEqual(fastsign(val), -1); counter += 1
        self.assertEqual(fastsign(sympy.Rational(3, 4)), 1); counter += 1
        self.assertTrue(isnan(fastsign(sympy.nan))); counter += 1
        self.assertEqual(fastsign(MyNumber(-5)), -1); counter += 1
        self.assertEqual(fastsign(MyNumber(-1)), -1); counter += 1
        self.assertEqual(fastsign(MyNumber(0)), 0); counter += 1
        self.assertEqual(fastsign(MyNumber(1)), 1); counter += 1
        self.assertEqual(fastsign(MyNumber(5.1)), 1); counter += 1
        with self.assertRaisesRegex(TypeError, r"must be real number, not MyNumber"):
            fastsign(MyNumber(nan))
        counter += 1
        with self.assertRaisesRegex(TypeError, r"signum\.fastsign\(\) takes exactly one argument \(0 given\)"):
            fastsign()
        counter += 1
        with self.assertRaisesRegex(TypeError, r"signum\.fastsign\(\) takes no keyword arguments"):
            fastsign(if_exc=None)
        counter += 1
        with self.assertRaisesRegex(TypeError, r"signum\.fastsign\(\) takes exactly one argument \(2 given\)"):
            fastsign(-1, 0)
        counter += 1
        with self.assertRaisesRegex(TypeError, r"signum\.fastsign\(\) takes exactly one argument \(3 given\)"):
            fastsign(-1, 0, 1)
        counter += 1
        with self.assertRaisesRegex(TypeError, r"signum\.fastsign\(\) takes no keyword arguments"):
            fastsign(5.0, code_shift=2)
        counter += 1
        with self.assertRaisesRegex(RuntimeError, r"Boom!"):
            fastsign(ExplodingNumber(-3.14))
        counter += 1
        self.assertEqual(fastsign(NotImplementedNumber(-3.14)), -1); counter += 1
        with self.assertRaisesRegex(TypeError, r"must be real number, not NoneType"):
            fastsign(None)
        counter += 1
        with self.assertRaisesRegex(TypeError, r"signum\.fastsign\(\): cannot compare or check for NaN\. Cause: comparison with 'int' not implemented for type 'str'"):
            fastsign('5.0')
        counter += 1
        with self.assertRaisesRegex(TypeError, r"signum\.fastsign\(\): cannot compare or check for NaN\. Cause: comparison with 'int' not implemented for type 'str'"):
            fastsign('nan')
        counter += 1
        with self.assertRaisesRegex(TypeError, r"signum\.fastsign\(\): cannot compare or check for NaN\. Cause: comparison with 'int' not implemented for type 'str'"):
            fastsign('number 5')
        counter += 1
        with self.assertRaisesRegex(TypeError, r"must be real number, not complex"):
            fastsign(-1+1j)
        counter += 1
        with self.assertRaisesRegex(TypeError, r"must be real number, not list"):
            fastsign([-8.75,])
        counter += 1
        with self.assertRaisesRegex(TypeError, r"must be real number, not set"):
            fastsign({-3.14,})
        counter += 1
        self.buffer.append(trace(prev_counter, counter, s_cnt, what="fastsign"))

        # fastsign vs sign: same results
        s_cnt += 1; prev_counter = counter
        x_sym = sympy.Symbol('x')
        expr = x_sym
        val = expr.subs(x_sym, -3.14)
        args = [
            -5,
            -1,
            0,
            1,
            5,
            True,
            False,
            10**1000,
            -10**1000,
            10**1000-10**1000,
            -5.0,
            -1.0,
            0.0,
            1.0,
            5.0,
            float('-0.0'),
            float('+0.0'),
            -inf,
            inf,
            Fraction(-5, 2),
            Fraction(-1, 2),
            Fraction(0, 2),
            Fraction(1, 2),
            Fraction(5, 2),
            Fraction(1, 1 << 2000),
            float(Fraction(1, 1 << 2000)),
            Decimal(-5.5),
            Decimal(-1.5),
            Decimal(0.0),
            Decimal(1.5),
            Decimal(5.5),
            val,
            sympy.Rational(3, 4),
            MyNumber(-5),
            MyNumber(-1),
            MyNumber(0),
            MyNumber(1),
            MyNumber(5.1),
        ]
        for x in args:
            self.assertEqual(fastsign(x), sign(x)); counter += 1
        self.buffer.append(trace(prev_counter, counter, s_cnt, what="fastsign vs sign: same results"))

        # fastsign vs sign: same NaNs
        s_cnt += 1; prev_counter = counter
        args = [
            float('-nan'),
            nan,
            0.0*nan,
            Decimal('NaN'),
            sympy.nan,
        ]
        for x in args:
            self.assertTrue(isnan(fastsign(x)) and isnan(sign(x))); counter += 1
        self.buffer.append(trace(prev_counter, counter, s_cnt, what="fastsign vs sign: same NaNs"))

        # fastsign vs sign: same error cases
        s_cnt += 1; prev_counter = counter
        args = [
            (TypeError, MyNumber(nan),),
            (RuntimeError, ExplodingNumber(-3.14),),
            (TypeError, None,),
            (TypeError, '5.0',),
            (TypeError, 'nan',),
            (TypeError, 'number 5',),
            (TypeError, -1+1j,),
            (TypeError, [-8.75,],),
            (TypeError, {-3.14,},),
        ]
        for t, x in args:
            with self.assertRaises(t):
              fastsign(x)
            self.assertEqual(sign(x, 2), 0)
            counter += 1
        self.buffer.append(trace(prev_counter, counter, s_cnt, what="fastsign vs sign: same error cases"))

        # fastsign vs sign: different behavior
        s_cnt += 1; prev_counter = counter
        x = NotImplementedNumber(-3.14)
        self.assertEqual(fastsign(x), -1)
        self.assertEqual(sign(x, 2), 0)
        counter += 1
        self.buffer.append(trace(prev_counter, counter, s_cnt, what="fastsign vs sign: different behavior"))

        self.buffer.append(f'\n{success(counter, s_cnt=s_cnt)}\n')
        print('\n'.join(self.buffer), flush=True)

if __name__ == '__main__':
    # Switch sys.stdout and sys.stderr to 'utf-8' encoding
    outflows = OutputUTF8()
    outflows.set_utf8()

    print(f'***** Test: {__file__}\n', flush=True)
    unittest.main()

    # Restore stdout and stderr
    outflows.reset_from_utf8()



from signum import sign as sign, fastsign as fastsign
from testing import get_passes, set_high_priority, success

from math import nan
import os
import psutil
import time

MAX_PASSES = get_passes(__file__)

def benchmark_and_leak_check():
    process = psutil.Process(os.getpid())

    # Test data
    test_cases = [
        ("Base (int)", lambda: sign(-5)),
        ("Fast (int)", lambda: fastsign(-5)),
        ("Base (float)", lambda: sign(-5.5)),
        ("Fast (float)", lambda: fastsign(-5.5)),
        ("Preprocess (str)", lambda: sign("15 men", preprocess=lambda s: (float(s.split()[0]),))),
        ("If_exc (error)", lambda: sign("error", if_exc=(-2,))),
        ("Codeshift (error)", lambda: sign("error", codeshift=2)),
        ("Codeshift (int)", lambda: sign(-5, codeshift=2)),
        ("Codeshift (nan)", lambda: sign(nan, codeshift=2)),
    ]

    print(f"{'Test Case':<20} | {'Time (s)':<10} | {'Memory (MB)':<12}")
    print("-" * 53)

    for name, func in test_cases:
        # Before
        start_mem = process.memory_info().rss / 1024 / 1024
        start_time = time.perf_counter()

        # Main loop (MAX_PASSES calls)
        for _ in range(MAX_PASSES):
            func()

        # After
        end_time = time.perf_counter()
        end_mem = process.memory_info().rss / 1024 / 1024

        duration = end_time - start_time
        mem_diff = end_mem - start_mem

        print(f"{name:<20} | {duration:>9.4f} | {end_mem:>10.2f} ({mem_diff:+.2f})")

if __name__ == "__main__":

    print(f'***** Test: {__file__}')
    print(f'MAX_PASSES: {MAX_PASSES}')
    print(f'*** {set_high_priority()} ***\n')

    for _ in range(MAX_PASSES): # Warm up Python
        sign(1)
    benchmark_and_leak_check()
    print(f'\n{success(9, passes=MAX_PASSES)}')



from signum import sign

from testing import SHORT_SIMPLE_TEST, SST_DICT, get_passes, set_high_priority, detect_version, success, \
                    open_devnull, close_devnull, OutputUTF8, \
                    EPS as _EPS, PIRATES as _PIRATES, n_extract as _n_extract, c_prep as _c_prep, \
                    MyNumber as _MyNumber, ExplodingNumber as _ExplodingNumber, \
                    NotImplementedNumber as _NotImplementedNumber, trace as _trace

from decimal import Decimal
from fractions import Fraction
from math import nan, inf
import sympy
import sys
import time

MAX_PASSES = get_passes(__file__)
_SST = min(SHORT_SIMPLE_TEST, SST_DICT.get(detect_version(), 'default'))

outflows = OutputUTF8()
outflows.set_utf8()

start_time = None
out_test   = None

print(f'***** Test: {__file__}')
print(f'MAX_PASSES: {MAX_PASSES}')
print(f'*** {set_high_priority()} ***\n')
for _ in range(MAX_PASSES + 1):
    s_cnt = 0
    counter = 0

    s_cnt += 1; prev_counter = counter
    print(f'{s_cnt:2} --- int', file=out_test)
    print("sign(-5):", sign(-5), file=out_test); counter += 1
    print("sign(-1):", sign(-1), file=out_test); counter += 1
    print("sign(0):", sign(0), file=out_test); counter += 1
    print("sign(1):", sign(1), file=out_test); counter += 1
    print("sign(5):", sign(5), file=out_test); counter += 1

    print(_trace(prev_counter, counter, s_cnt), file=out_test)

    s_cnt += 1; prev_counter = counter
    print(f'\n{s_cnt:2} --- bool', file=out_test)
    print("sign(True):", sign(True), file=out_test); counter += 1
    print("sign(False):", sign(False), file=out_test); counter += 1

    print(_trace(prev_counter, counter, s_cnt), file=out_test)

    s_cnt += 1; prev_counter = counter
    print(f'\n{s_cnt:2} --- big numbers', file=out_test)
    print('sign(10**1000):', sign(10**1000), file=out_test); counter += 1
    print('sign(-10**1000):', sign(-10**1000), file=out_test); counter += 1
    print('sign(10**1000-10**1000):', sign(10**1000-10**1000), file=out_test); counter += 1

    print(_trace(prev_counter, counter, s_cnt), file=out_test)

    s_cnt += 1; prev_counter = counter
    print(f'\n{s_cnt:2} --- float', file=out_test)
    print("sign(-5.0):", sign(-5.0), file=out_test); counter += 1
    print("sign(-1.0):", sign(-1.0), file=out_test); counter += 1
    print("sign(0.0):", sign(0.0), file=out_test); counter += 1
    print("sign(1.0):", sign(1.0), file=out_test); counter += 1
    print("sign(5.0):", sign(5.0), file=out_test); counter += 1

    print(_trace(prev_counter, counter, s_cnt), file=out_test)

    s_cnt += 1; prev_counter = counter
    print(f'\n{s_cnt:2} --- -0.0 and +0.0', file=out_test)
    print("sign(float('-0.0')):", sign(float('-0.0')), file=out_test); counter += 1
    print("sign(float('+0.0')):", sign(float('+0.0')), file=out_test); counter += 1

    print(_trace(prev_counter, counter, s_cnt), file=out_test)

    s_cnt += 1; prev_counter = counter
    print(f'\n{s_cnt:2} --- -inf and inf', file=out_test)
    print("sign(-inf):", sign(-inf), file=out_test); counter += 1
    print("sign(inf):", sign(inf), file=out_test); counter += 1

    print(_trace(prev_counter, counter, s_cnt), file=out_test)

    s_cnt += 1; prev_counter = counter
    print(f'\n{s_cnt:2} --- -nan and nan', file=out_test)
    print("sign(float('-nan')):", sign(float('-nan')), file=out_test); counter += 1
    print("sign(nan):", sign(nan), file=out_test); counter += 1
    print("sign(0.0*nan):", sign(0.0*nan), file=out_test); counter += 1

    print(_trace(prev_counter, counter, s_cnt), file=out_test)

    s_cnt += 1; prev_counter = counter
    print(f'\n{s_cnt:2} --- Fraction', file=out_test)
    print("sign(Fraction(-5, 2)):", sign(Fraction(-5, 2)), file=out_test); counter += 1
    print("sign(Fraction(-1, 2)):", sign(Fraction(-1, 2)), file=out_test); counter += 1
    print("sign(Fraction(0, 2)):", sign(Fraction(0, 2)), file=out_test); counter += 1
    print("sign(Fraction(1, 2)):", sign(Fraction(1, 2)), file=out_test); counter += 1
    print("sign(Fraction(5, 2)):", sign(Fraction(5, 2)), file=out_test); counter += 1

    print(_trace(prev_counter, counter, s_cnt), file=out_test)

    s_cnt += 1; prev_counter = counter
    print(f'\n{s_cnt:2} --- Tests with a very small argument kindly provided by Tim Peters', file=out_test)
    print(f"{sign(Fraction(1, 1 << 2000))  =  }".replace('  =  ', ': ', 1), file=out_test); counter += 1
    print(f"{sign(float(Fraction(1, 1 << 2000)))  =  }".replace('  =  ', ': ', 1), file=out_test); counter += 1

    print(_trace(prev_counter, counter, s_cnt), file=out_test)

    s_cnt += 1; prev_counter = counter
    print(f'\n{s_cnt:2} --- Decimal', file=out_test)
    print("sign(Decimal(-5.5)):", sign(Decimal(-5.5)), file=out_test); counter += 1
    print("sign(Decimal(-1.5)):", sign(Decimal(-1.5)), file=out_test); counter += 1
    print("sign(Decimal(0.0)):", sign(Decimal(0.0)), file=out_test); counter += 1
    print("sign(Decimal(1.5)):", sign(Decimal(1.5)), file=out_test); counter += 1
    print("sign(Decimal(5.5)):", sign(Decimal(5.5)), file=out_test); counter += 1

    print(_trace(prev_counter, counter, s_cnt), file=out_test)

    s_cnt += 1; prev_counter = counter
    print(f"\n{s_cnt:2} --- Decimal('NaN')", file=out_test)
    print("sign(Decimal('NaN')):", sign(Decimal('NaN')), file=out_test); counter += 1

    print(_trace(prev_counter, counter, s_cnt), file=out_test)

    s_cnt += 1; prev_counter = counter
    print(f'\n{s_cnt:2} --- sympy (substitution and Rational)', file=out_test)
    x_sym = sympy.Symbol('x')
    expr = x_sym
    val = expr.subs(x_sym, -3.14)
    print(f"val: {repr(val)}; type(val): {type(val)}", file=out_test)
    print(f"type(val > 0): {type(val > 0)}", file=out_test)
    print("sign(val):", sign(val), file=out_test); counter += 1
    print("sign(sympy.Rational(3, 4)):", sign(sympy.Rational(3, 4)), file=out_test); counter += 1

    print(_trace(prev_counter, counter, s_cnt), file=out_test)

    s_cnt += 1; prev_counter = counter
    print(f'\n{s_cnt:2} --- sympy.nan', file=out_test)
    print("sign(sympy.nan):", sign(sympy.nan), file=out_test); counter += 1

    print(_trace(prev_counter, counter, s_cnt), file=out_test)

    s_cnt += 1; prev_counter = counter
    print(f'\n{s_cnt:2} --- My Custom Class That Have >, <, == With Numbers But Nothing Else', file=out_test)
    print("sign(_MyNumber(-5)):",  sign(_MyNumber(-5)), file=out_test); counter += 1
    print("sign(_MyNumber(-1)):",  sign(_MyNumber(-1)), file=out_test); counter += 1
    print("sign(_MyNumber(0)):",   sign(_MyNumber(0)), file=out_test); counter += 1
    print("sign(_MyNumber(1)):",   sign(_MyNumber(1)), file=out_test); counter += 1
    print("sign(_MyNumber(5.1)):", sign(_MyNumber(5.1)), file=out_test); counter += 1
    try:
        print("sign(_MyNumber(nan)):", sign(_MyNumber(nan)), file=out_test)
    except TypeError as e:
        print(f"- {type(e).__name__}: {e}", file=out_test)
    finally:
        counter += 1

    print(_trace(prev_counter, counter, s_cnt), file=out_test)

    s_cnt += 1; prev_counter = counter
    new_test = '0 with keys, ' if _SST > 0 else ''
    print(f"\n{s_cnt:2} --- invalid number of positional arguments (0, {new_test}2, 3, 4)", file=out_test)
    try:
        print("sign():", sign(), file=out_test)
    except TypeError as e:
        print(f"- {type(e).__name__}: {e}", file=out_test)
    finally:
        counter += 1

    if _SST > 0: # This test is for v1.1.0+ only
        try:
            print("sign(preprocess=lambda a: (float(a),), if_exc=None):",
                  sign(preprocess=lambda a: (float(a),), if_exc=None), file=out_test)
        except TypeError as e:
            print(f"- {type(e).__name__}: {e}", file=out_test)
        finally:
            counter += 1

    if _SST < 3:
        try:
            print("sign(-1, 0):", sign(-1, 0), file=out_test)
        except TypeError as e:
            print(f"- {type(e).__name__}: {e}", file=out_test)
        finally:
            counter += 1

    try:
        print("sign(-1, 0, 1):", sign(-1, 0, 1), file=out_test)
    except TypeError as e:
        print(f"- {type(e).__name__}: {e}", file=out_test)
    finally:
        counter += 1

    try:
        print("sign(-1, 0, 1, 4):", sign(-1, 0, 1, 4), file=out_test)
    except TypeError as e:
        print(f"- {type(e).__name__}: {e}", file=out_test)
    finally:
        counter += 1

    try:
        print("sign(-1, 0, 1, 4, 5):", sign(-1, 0, 1, 4, 5), file=out_test)
    except TypeError as e:
        print(f"- {type(e).__name__}: {e}", file=out_test)
    finally:
        counter += 1

    try:
        print("sign(5.0, code_shift=2):", sign(5.0, code_shift=2), file=out_test)
    except TypeError as e:
        print(f"- {type(e).__name__}: {e}", file=out_test)
    finally:
        counter += 1

    print(_trace(prev_counter, counter, s_cnt), file=out_test)

    s_cnt += 1; prev_counter = counter
    print(f'\n{s_cnt:2} --- ExplodingNumber, NotImplementedNumber', file=out_test)
    try:
        print("sign(_ExplodingNumber(-3.14):", sign(_ExplodingNumber(-3.14)), file=out_test)
    except TypeError as e:
        print(f"- {type(e).__name__}: {e}", file=out_test)
    finally:
        counter += 1

    try:
        print("sign(_NotImplementedNumber(-3.14):", sign(_NotImplementedNumber(-3.14)), file=out_test)
    except TypeError as e:
        print(f"- {type(e).__name__}: {e}", file=out_test)
    finally:
        counter += 1

    print(_trace(prev_counter, counter, s_cnt), file=out_test)

    s_cnt += 1; prev_counter = counter
    print(f"\n{s_cnt:2} --- inappropriate argument types (None, str, complex, list, set)", file=out_test)
    tests = [None, '5.0', 'nan', 'number 5', -1+1j, [-8.75], {-3.14},]
    for x in tests:
        try:
            print("sign({repr(x)}):", sign(x), file=out_test)
        except TypeError as e:
            print(f"- {type(e).__name__}: {e}", file=out_test)
        finally:
            counter += 1

    print(_trace(prev_counter, counter, s_cnt), file=out_test)

    # New options since version 1.1.0; skip these tests for v1.0.2

    if _SST > 0: # v1.1.0+
        s_cnt += 1; prev_counter = counter
        print(f'\n{s_cnt:2} --- preprocess key, simple argument replacement, string conversion', file=out_test)
        tests = ['5.0', 'nan', -18]
        for x in tests:
            try:
                print(f"sign({repr(x)}, "
                      f"preprocess=lambda a: (float(a),)):", sign(x, preprocess=lambda a: (float(a),)), file=out_test)
            except TypeError as e:
                print(f"- {type(e).__name__}: {e}", file=out_test)
            finally:
                counter += 1

        print(_trace(prev_counter, counter, s_cnt), file=out_test)

        s_cnt += 1; prev_counter = counter
        print(f'\n{s_cnt:2} --- preprocess key, argument replacement, treat small number as zero', file=out_test)
        tests = [-1, 0, -.187e-17, 5.0]
        for x in tests:
            print(f"sign({x}, "
                  f"preprocess=lambda a: (0 if abs(a) < _EPS else a,)):",
                  sign(x, preprocess=lambda a: (0 if abs(a) < _EPS else a,)), file=out_test); counter += 1

        print(_trace(prev_counter, counter, s_cnt), file=out_test)

        s_cnt += 1; prev_counter = counter
        print(f'\n{s_cnt:2} --- preprocess key, replace only string argument, extract number from string', file=out_test)

        tests = [_PIRATES, 'Temperature is -.12e+02 °C', 'error', 123]
        for x in tests:
            try:
                print(f"sign({repr(x)}, "
                      f"preprocess=n_extract):", sign(x, preprocess=_n_extract), file=out_test)
            except TypeError as e:
                print(f"- {type(e).__name__}: {e}", file=out_test)
            finally:
                counter += 1

        print(_trace(prev_counter, counter, s_cnt), file=out_test)

        s_cnt += 1; prev_counter = counter
        print(f'\n{s_cnt:2} --- preprocess key, replace result, permits complex arguments', file=out_test)
        tests = [-1+1j, -18.4]

        for x in tests:
            try:
                print(f"sign({repr(x)}, "
                      f"preprocess=c_prep):", sign(x, preprocess=_c_prep), file=out_test)
            except TypeError as e:
                print(f"- {type(e).__name__}: {e}", file=out_test)
            finally:
                counter += 1

        print(_trace(prev_counter, counter, s_cnt), file=out_test)

        s_cnt += 1; prev_counter = counter
        print(f"\n{s_cnt:2} --- preprocess key, replace result, float result for 'float' and 'Decimal', sign recursion", file=out_test)
        tests = [-5, -5.0, Decimal(-5.5)]
        for x in tests:
            try:
                print(f"sign({repr(x)}, "
                      f"preprocess=lambda a: (a, float(sign(a))) if isinstance(a, (float, Decimal)) else None):",
                      sign(x, preprocess=lambda a: (a, float(sign(a))) if isinstance(a, (float, Decimal)) else None), file=out_test)
            except TypeError as e:
                print(f"- {type(e).__name__}: {e}", file=out_test)
            finally:
                counter += 1

        print(_trace(prev_counter, counter, s_cnt), file=out_test)

        s_cnt += 1; prev_counter = counter
        print(f'\n{s_cnt:2} --- preprocess key, replace result or argument, treat small number as zero differently', file=out_test)
        tests = [-1, 0, -.187e-17, 5.0]
        ppl = lambda x: (x, 0) if abs(x) < _EPS else (x,)
        for x in tests:
            print(f"sign({x}, preprocess=ppl):", sign(x, preprocess=ppl), file=out_test); counter += 1

        print(_trace(prev_counter, counter, s_cnt), file=out_test)

        s_cnt += 1; prev_counter = counter
        print(f'\n{s_cnt:2} --- if_exc key (exception safety)', file=out_test)
        tests = [(None, '?'), ('5.0', '?'), ('nan', '?'), ('number 5', '?'),
                 (-1+1j, '?'), ([-8.75], '?'), ({-3.14}, '?'), # blocking exception, marked '?'
                 (-1, '!'), (31.4, '!'), (nan, '!'), (Fraction(-99, 19), '!'), (Decimal('101.78'), '!'),]                                              # valid nuneric types, marked '!'
        flag = 0
        repl = [None, -2, nan, None,]
        nrepl = len(repl)
        sp = ' '; quo = "'"
        for x, mark in tests:
            try:
                print(f"Type: {quo + type(x).__name__ + quo:10} {mark} "
                      f"sign({repr(x)}, "
                      f"if_exc=({repl[flag]},)):",
                      sign(x, if_exc=(repl[flag],)), file=out_test)
            except TypeError as e:
                print(f"- {type(e).__name__}: {e}", file=out_test)
            finally:
                counter += 1
            flag = (flag + 1) % nrepl

        print(_trace(prev_counter, counter, s_cnt), file=out_test)

        s_cnt += 1; prev_counter = counter
        print(f"\n{s_cnt:2} --- both preprocess and if_exc key", file=out_test)
        tests = [-5, -5.0, Decimal(-5.5), 'error']
        for x in tests:
            try:
                print(f"sign({repr(x)}, "
                      f"preprocess=lambda a: (a, float(sign(a))) if isinstance(a, (float, Decimal)) else None, "
                      f"if_exc=(None,)):",
                      sign(x, preprocess=lambda a: (a, float(sign(a))) if isinstance(a, (float, Decimal)) else None, if_exc=(None,)), file=out_test)
            except TypeError as e:
                print(f"- {type(e).__name__}: {e}", file=out_test)
            finally:
                counter += 1

        print(_trace(prev_counter, counter, s_cnt), file=out_test)

    # New options since version 1.2.0; skip these tests for v1.0.2, v1.1.0+

    if _SST > 1: # v1.2.0+
        s_cnt += 1; prev_counter = counter
        print(f"\n{s_cnt:2} --- codeshift and key combinations", file=out_test)
        tests = ['error', -5, 0, 5, nan]
        for x in tests:
            print(f"sign({repr(x)}, codeshift=2):",
                  sign(x, codeshift=2), file=out_test); counter += 1
        print('   ---', file=out_test)
        for x in tests:
            print(f"sign({repr(x)}, if_exc=(-7,), codeshift=2):",
                  sign(x, if_exc=(-7,), codeshift=2), file=out_test); counter += 1
        print('   ---', file=out_test)
        tests = ['error', -1, 0, -.187e-17, 5.0]
        for x in tests:
            print(f"sign({repr(x)}, "
                  f"preprocess=lambda a: (0 if abs(a) < _EPS else a,), codeshift=2):",
                  sign(x, preprocess=lambda a: (0 if abs(a) < _EPS else a,), codeshift=2),
                  file=out_test); counter += 1
        print('   ---', file=out_test)
        tests = ['error', -5.0, 0.0, 0, 5, nan]
        for x in tests:
            print(f"sign({repr(x)}, "
                  f"preprocess=lambda a: (a, float(sign(a))) if isinstance(a, float) else None, "
                  f"codeshift=1):",
                  sign(x, preprocess=lambda a: (a, float(sign(a))) if isinstance(a, float) else None, codeshift=1), file=out_test); counter += 1
        print('   ---', file=out_test)
        for x in tests:
            print(f"sign({repr(x)}, "
                  f"preprocess=lambda a: (a, float(sign(a))) if isinstance(a, float) else None, "
                  f"if_exc=(-7,), codeshift=1):",
                  sign(x, preprocess=lambda a: (a, float(sign(a))) if isinstance(a, float) else None, if_exc=(-7,), codeshift=1), file=out_test); counter += 1

        print(_trace(prev_counter, counter, s_cnt), file=out_test)

    if _SST > 2: # This is for v.1.2.3+
        # v1.2.3: positional codeshift and fastsign
        from signum import fastsign

        s_cnt += 1; prev_counter = counter
        print(f"\n{s_cnt:2} --- v1.2.3: positional codeshift", file=out_test)
        try:
            print("sign(-77, None, codeshift=2):", sign(-77, None, codeshift=2), file=out_test)
        except TypeError as e:
            print(f"- {type(e).__name__}: {e}", file=out_test)
        finally:
            counter += 1
        tests = ['error', -5, 0, 5, nan]
        for x in tests:
            print(f"sign({repr(x)}, 2):",
                  sign(x, 2), file=out_test); counter += 1
        print('   ---', file=out_test)
        for x in tests:
            print(f"sign({repr(x)}, 2, if_exc=(-7,)):",
                  sign(x, 2, if_exc=(-7,)), file=out_test); counter += 1
        print('   ---', file=out_test)
        tests = ['error', -1, 0, -.187e-17, 5.0]
        for x in tests:
            print(f"sign({repr(x)}, 2, "
                  f"preprocess=lambda a: (0 if abs(a) < _EPS else a,)):",
                  sign(x, 2, preprocess=lambda a: (0 if abs(a) < _EPS else a,)),
                  file=out_test); counter += 1
        print('   ---', file=out_test)
        tests = ['error', -5.0, 0.0, 0, 5, nan]
        for x in tests:
            print(f"sign({repr(x)}, 1, "
                  f"preprocess=lambda a: (a, float(sign(a))) if isinstance(a, float) else None):",
                  sign(x, 1, preprocess=lambda a: (a, float(sign(a))) if isinstance(a, float) else None), file=out_test); counter += 1
        print(f"sign({repr(x)}, 1, "
              f"preprocess=lambda a: (a, float(sign(a))) if isinstance(a, float) and not isnan(a) else None):",
              sign(x, 1, preprocess=lambda a: (a, float(sign(a))) if isinstance(a, float) and not isnan(a) else None), file=out_test); counter += 1
        print('   ---', file=out_test)
        for x in tests:
            print(f"sign({repr(x)}, 1, "
                  f"preprocess=lambda a: (a, float(sign(a))) if isinstance(a, float) else None, "
                  f"if_exc=(-7,)):",
                  sign(x, 1, preprocess=lambda a: (a, float(sign(a))) if isinstance(a, float) else None, if_exc=(-7,)), file=out_test); counter += 1
        print(f"sign({repr(x)}, 1, "
              f"preprocess=lambda a: (a, float(sign(a))) if isinstance(a, float) and not isnan(a) else None, "
              f"if_exc=(-7,)):",
              sign(x, 1, preprocess=lambda a: (a, float(sign(a))) if isinstance(a, float) and not isnan(a) else None, if_exc=(-7,)), file=out_test); counter += 1

        print(_trace(prev_counter, counter, s_cnt), file=out_test)

        s_cnt += 1; prev_counter = counter
        print(f'\n{s_cnt:2} --- v1.2.3: fastsign', file=out_test)
        print("fastsign(-5):", fastsign(-5), file=out_test); counter += 1
        print("fastsign(-1):", fastsign(-1), file=out_test); counter += 1
        print("fastsign(0):", fastsign(0), file=out_test); counter += 1
        print("fastsign(1):", fastsign(1), file=out_test); counter += 1
        print("fastsign(5):", fastsign(5), file=out_test); counter += 1
        print('   ---', file=out_test)
        print("fastsign(True):", fastsign(True), file=out_test); counter += 1
        print("fastsign(False):", fastsign(False), file=out_test); counter += 1
        print('   ---', file=out_test)
        print('fastsign(10**1000):', fastsign(10**1000), file=out_test); counter += 1
        print('fastsign(-10**1000):', fastsign(-10**1000), file=out_test); counter += 1
        print('fastsign(10**1000-10**1000):', fastsign(10**1000-10**1000), file=out_test); counter += 1
        print('   ---', file=out_test)
        print("fastsign(-5.0):", fastsign(-5.0), file=out_test); counter += 1
        print("fastsign(-1.0):", fastsign(-1.0), file=out_test); counter += 1
        print("fastsign(0.0):", fastsign(0.0), file=out_test); counter += 1
        print("fastsign(1.0):", fastsign(1.0), file=out_test); counter += 1
        print("fastsign(5.0):", fastsign(5.0), file=out_test); counter += 1
        print('   ---', file=out_test)
        print("fastsign(float('-0.0')):", fastsign(float('-0.0')), file=out_test); counter += 1
        print("fastsign(float('+0.0')):", fastsign(float('+0.0')), file=out_test); counter += 1
        print('   ---', file=out_test)
        print("fastsign(-inf):", fastsign(-inf), file=out_test); counter += 1
        print("fastsign(inf):", fastsign(inf), file=out_test); counter += 1
        print('   ---', file=out_test)
        print("fastsign(float('-nan')):", fastsign(float('-nan')), file=out_test); counter += 1
        print("fastsign(nan):", fastsign(nan), file=out_test); counter += 1
        print("fastsign(0.0*nan):", fastsign(0.0*nan), file=out_test); counter += 1
        print('   ---', file=out_test)
        print("fastsign(Fraction(-5, 2)):", fastsign(Fraction(-5, 2)), file=out_test); counter += 1
        print("fastsign(Fraction(-1, 2)):", fastsign(Fraction(-1, 2)), file=out_test); counter += 1
        print("fastsign(Fraction(0, 2)):", fastsign(Fraction(0, 2)), file=out_test); counter += 1
        print("fastsign(Fraction(1, 2)):", fastsign(Fraction(1, 2)), file=out_test); counter += 1
        print("fastsign(Fraction(5, 2)):", fastsign(Fraction(5, 2)), file=out_test); counter += 1
        print('   ---', file=out_test)
        print(f"{fastsign(Fraction(1, 1 << 2000))  =  }".replace('  =  ', ': ', 1), file=out_test); counter += 1
        print(f"{fastsign(float(Fraction(1, 1 << 2000)))  =  }".replace('  =  ', ': ', 1), file=out_test); counter += 1
        print('   ---', file=out_test)
        print("fastsign(Decimal(-5.5)):", fastsign(Decimal(-5.5)), file=out_test); counter += 1
        print("fastsign(Decimal(-1.5)):", fastsign(Decimal(-1.5)), file=out_test); counter += 1
        print("fastsign(Decimal(0.0)):", fastsign(Decimal(0.0)), file=out_test); counter += 1
        print("fastsign(Decimal(1.5)):", fastsign(Decimal(1.5)), file=out_test); counter += 1
        print("fastsign(Decimal(5.5)):", fastsign(Decimal(5.5)), file=out_test); counter += 1
        print('   ---', file=out_test)
        print("fastsign(Decimal('NaN')):", fastsign(Decimal('NaN')), file=out_test); counter += 1
        print('   ---', file=out_test)
        x_sym = sympy.Symbol('x')
        expr = x_sym
        val = expr.subs(x_sym, -3.14)
        print(f"val: {repr(val)}; type(val): {type(val)}", file=out_test)
        print(f"type(val > 0): {type(val > 0)}", file=out_test)
        print("fastsign(val):", fastsign(val), file=out_test); counter += 1
        print("fastsign(sympy.Rational(3, 4)):", fastsign(sympy.Rational(3, 4)), file=out_test); counter += 1
        print('   ---', file=out_test)
        print("fastsign(sympy.nan):", fastsign(sympy.nan), file=out_test); counter += 1
        print('   ---', file=out_test)
        print("fastsign(_MyNumber(-5)):",  fastsign(_MyNumber(-5)), file=out_test); counter += 1
        print("fastsign(_MyNumber(-1)):",  fastsign(_MyNumber(-1)), file=out_test); counter += 1
        print("fastsign(_MyNumber(0)):",   fastsign(_MyNumber(0)), file=out_test); counter += 1
        print("fastsign(_MyNumber(1)):",   fastsign(_MyNumber(1)), file=out_test); counter += 1
        print("fastsign(_MyNumber(5.1)):", fastsign(_MyNumber(5.1)), file=out_test); counter += 1
        try:
            print("fastsign(_MyNumber(nan)):", fastsign(_MyNumber(nan)), file=out_test)
        except TypeError as e:
            print(f"- {type(e).__name__}: {e}", file=out_test)
        finally:
            counter += 1
        print('   ---', file=out_test)
        try:
            print("fastsign():", fastsign(), file=out_test)
        except TypeError as e:
            print(f"- {type(e).__name__}: {e}", file=out_test)
        finally:
            counter += 1
        try:
            print("fastsign(if_exc=None):",
                  fastsign(if_exc=None), file=out_test)
        except TypeError as e:
            print(f"- {type(e).__name__}: {e}", file=out_test)
        finally:
            counter += 1
        try:
            print("fastsign(-1, 0):", fastsign(-1, 0), file=out_test)
        except TypeError as e:
            print(f"- {type(e).__name__}: {e}", file=out_test)
        finally:
            counter += 1
        try:
            print("fastsign(-1, 0, 1):", fastsign(-1, 0, 1), file=out_test)
        except TypeError as e:
            print(f"- {type(e).__name__}: {e}", file=out_test)
        finally:
            counter += 1
        try:
            print("fastsign(5.0, code_shift=2):", fastsign(5.0, code_shift=2), file=out_test)
        except TypeError as e:
            print(f"- {type(e).__name__}: {e}", file=out_test)
        finally:
            counter += 1
        print('   ---', file=out_test)
        try:
            print("fastsign(_ExplodingNumber(-3.14):", fastsign(_ExplodingNumber(-3.14)), file=out_test)
        except (TypeError, RuntimeError) as e:
            print(f"- {type(e).__name__}: {e}", file=out_test)
        finally:
            counter += 1
        print('   ---', file=out_test)
        try:
            print("fastsign(_NotImplementedNumber(-3.14):", fastsign(_NotImplementedNumber(-3.14)), file=out_test)
        except TypeError as e:
            print(f"- {type(e).__name__}: {e}", file=out_test)
        finally:
            counter += 1
        print('   ---', file=out_test)
        tests = [None, '5.0', 'nan', 'number 5', -1+1j, [-8.75,], {-3.14,},]
        for x in tests:
            try:
                print("fastsign({repr(x)}):", fastsign(x), file=out_test)
            except TypeError as e:
                print(f"- {type(e).__name__}: {e}", file=out_test)
            finally:
                counter += 1
        print(_trace(prev_counter, counter, s_cnt), file=out_test)

    if start_time is None: # The very first pass to warm Python
        start_time = time.perf_counter()
        # Block printing
        out_test = open_devnull()

# Finalize out_test
close_devnull(out_test)
out_test = None

print(f'\n{success(counter, s_cnt=s_cnt, start_time=start_time, passes=MAX_PASSES)}')

# Restore stdout and stderr
outflows.reset_from_utf8()



from signum import fastsign
from testing import get_passes, set_high_priority, success, OutputUTF8
from testing import MyNumber as _MyNumber, ExplodingNumber as _ExplodingNumber, NotImplementedNumber as _NotImplementedNumber

from decimal import Decimal
from fractions import Fraction
from math import nan, inf
import sympy
import time

MAX_PASSES = get_passes(__file__)

# Switch sys.stdout and sys.stderr to 'utf-8' encoding
outflows = OutputUTF8()
outflows.set_utf8()

start_time = None

print(f'***** Test: {__file__}')
print(f'MAX_PASSES: {MAX_PASSES}')
print(f'*** {set_high_priority()} ***\n')
for _ in range(MAX_PASSES + 1):
    (fastsign(-5))
    (fastsign(-1))
    (fastsign(0))
    (fastsign(1))
    (fastsign(5))
    (fastsign(True))
    (fastsign(False))
    (fastsign(10**1000))
    (fastsign(-10**1000))
    (fastsign(10**1000-10**1000))
    (fastsign(-5.0))
    (fastsign(-1.0))
    (fastsign(0.0))
    (fastsign(1.0))
    (fastsign(5.0))
    (fastsign(float('-0.0')))
    (fastsign(float('+0.0')))
    (fastsign(-inf))
    (fastsign(inf))
    (fastsign(float('-nan')))
    (fastsign(nan))
    (fastsign(0.0*nan))
    (fastsign(Fraction(-5, 2)))
    (fastsign(Fraction(-1, 2)))
    (fastsign(Fraction(0, 2)))
    (fastsign(Fraction(1, 2)))
    (fastsign(Fraction(5, 2)))
    (fastsign(Decimal(-5.5)))
    (fastsign(Decimal(-1.5)))
    (fastsign(Decimal(0.0)))
    (fastsign(Decimal(1.5)))
    (fastsign(Decimal(5.5)))
    (fastsign(Decimal('NaN')))
    x_sym = sympy.Symbol('x')
    expr = x_sym
    val = expr.subs(x_sym, -3.14)
    (fastsign(val))
    (fastsign(sympy.Rational(3, 4)))
    (fastsign(sympy.nan))
    (fastsign(_MyNumber(-5)))
    (fastsign(_MyNumber(-1)))
    (fastsign(_MyNumber(0)))
    (fastsign(_MyNumber(1)))
    (fastsign(_MyNumber(5.1)))
    try:
        (fastsign(_MyNumber(nan)))
    except TypeError as e:
        (e)
    try:
        (fastsign())
    except TypeError as e:
        (e)
    try:
        (fastsign(-1, 0))
    except TypeError as e:
        (e)
    try:
        (fastsign(-1, 0, 1))
    except TypeError as e:
        (e)
    try:
        (fastsign(-1, 0, 1, 4))
    except TypeError as e:
        (e)
    try:
        (fastsign(-1, 0, 1, 4, 5))
    except TypeError as e:
        (e)
    try:
        (fastsign(5.0, code_shift=2))
    except TypeError as e:
        (e)
    try:
        (fastsign(_ExplodingNumber(-3.14)))
    except (TypeError, RuntimeError) as e:
        (e)
    try:
        (fastsign(_NotImplementedNumber(-3.14)))
    except TypeError as e:
        (e)
    tests = [None, '5.0', 'nan', 'number 5', -1+1j, [-8.75], {-3.14},]
    for x in tests:
        try:
            (fastsign(x))
        except TypeError as e:
            (e)

    if start_time is None: # The very first pass to warm Python
        start_time = time.perf_counter()

print(f'{success(57, s_cnt=None, start_time=start_time, passes=MAX_PASSES)}\n')

# Restore stdout and stderr
outflows.reset_from_utf8()



from setuptools import setup, Extension
from setuptools.command.build_ext import build_ext

copt = {
    'msvc':    ['/Ox', '/std:c++20'],
    'mingw32': ['-O3', '-std=c++20'],
    'unix':    ['-O3', '-std=c++20'],
}

class build_ext_subclass(build_ext):
    def build_extensions(self):
        if (c := self.compiler.compiler_type) in copt:
            for e in self.extensions:
                e.extra_compile_args = copt[c]
        super().build_extensions()

setup(
    name="signum",
    version="1.2.3",
    ext_modules=[Extension("signum", ["signum.cpp"])],
    cmdclass={'build_ext': build_ext_subclass},
    include_package_data=True,
    install_requires=[],
    extras_require={
        'test': [
            'psutil',
            'sympy',
        ],
    },
)
